% File name: compute_quad_lqr_k
% Author: Phillip Jones (7/26/2022)
% Description: Compute Quadcopter K for LQR control law for CrazyFlie

%  CrazyFlie constants:
g = 9.8;        % (m/s^2) Gravity
m = 23.1/1000;    % (Kg) Mass of quad (Crazyflie max take-off weight  ~42g) (Foster-2015)
len = (.092/2); % (m) Length of quad lever arm for yaw force (92mm diameter of CrazyFlie)
len_xy = (.092/2)*sin(pi/4); % (m) Length of quad lever arm for pitch/roll force: Quad arm length * projection onto pitch/roll axis
I_xx = 1.33e-5; % (Kg*m^2) Moment of inertia about X-axis (McInerney-MS-Thesis-2017)
I_yy = 1.33e-5; % (Kg*m^2) Moment of inertia about Y-axis (McInerney-MS-Thesis-2017)
I_zz = 2.64e-5; % (Kg*m^2) Moment of inertia about Z-axis (McInerney-MS-Thesis-2017)

%  Matrix A :
%     x   y   z   phi theta psi x_dot y_dot z_dot phi_dot theta_dot psi_dot    
%     x1  x2  x3  x4   x5   x6   x7    x8    x9     x10      x11    x12
A = [ 0   0   0   0    0    0    1     0     0      0        0      0; ...
      0   0   0   0    0    0    0     1     0      0        0      0; ...
      0   0   0   0    0    0    0     0     1      0        0      0; ...
      0   0   0   0    0    0    0     0     0      1        0      0; ...
      0   0   0   0    0    0    0     0     0      0        1      0; ...
      0   0   0   0    0    0    0     0     0      0        0      1; ...
      0   0   0   0   -g    0    0     0     0      0        0      0; ...
      0   0   0   g    0    0    0     0     0      0        0      0; ...
      0   0   0   0    0    0    0     0     0      0        0      0; ...
      0   0   0   0    0    0    0     0     0      0        0      0; ...
      0   0   0   0    0    0    0     0     0      0        0      0; ...
      0   0   0   0    0    0    0     0     0      0        0      0];

%   Matrix B :
%    Thurst  Phi force Theta force  Psi force
%     u1       u2        u3         u4
B = [ 0        0         0          0; ...    % x1_dot  (x_dot) 
      0        0         0          0; ...    % x2_dot  (y_dot)
      0        0         0          0; ...    % x3_dot  (z_dot)
      0        0         0          0; ...    % x4_dot  (phi_dot)
      0        0         0          0; ...    % x5_dot  (theta_dot)
      0        0         0          0; ...    % x6_dot  (psi_dot)
      0        0         0          0; ...    % x7_dot  (x_ddot)
      0        0         0          0; ...    % x8_dot  (y_ddot)
     -1/m      0         0          0; ...    % x9_dot  (z_ddot)
      0   len_xy/I_xx    0          0; ...    % x10_dot (phi_ddot)
      0        0     -len_xy/I_yy    0; ...    % x11_dot (theta_ddot)
      0        0         0       len/I_zz ];  % x12_dot (psi_ddot)


%  LQR: Q and R matrix (Note the Q and R matrix is differnt from the Q and
%       R in LQR.
%
Q = [ 1  0  0  0   0   0  0  0  0   0  0  0; ...
      0  1  0  0   0   0  0  0  0   0  0  0; ...
      0  0  1  0   0   0  0  0  0   0  0  0; ...
      0  0  0 .0000001 0   0  0  0  0   0  0  0; ...
      0  0  0  0 .0000001  0  0  0  0   0  0  0; ...
      0  0  0  0   0   0.0000001  0  0  0   0  0  0; ...
      0  0  0  0   0   0 2  0  0   0  0  0; ...
      0  0  0  0   0   0  0 2  0   0  0  0; ...
      0  0  0  0   0   0  0  0 2  0  0  0; ...
      0  0  0  0   0   0  0  0  0 .0000001  0  0; ...
      0  0  0  0   0   0  0  0  0   0 .0000001 0; ...
      0  0  0  0   0   0  0  0  0   0  0  0.0000001];

R = [1  0    0    0; ...
     0    1  0    0; ...
     0    0    1  0; ...
     0    0    0    10000000];

[K, S, E] = lqr(A,B,Q,R);

K

[dK, S, E] = lqrd(A,B,Q,R,.001);

dK

